from enum import Enum

from rooms_shared_services.src.storage.models import BaseDynamodbModel


class CurrencyCode(str, Enum):
    ILS = "ILS"
    EUR = "EUR"
    RUB = "RUB"
    USD = "USD"
    PLN = "PLN"
    GBP = "GBP"


class GeoRelatedValues(BaseDynamodbModel):
    IL: str | float | int | None = None



from uuid import UUID

from rooms_shared_services.src.models.products.attributes import ProductAttribute
from rooms_shared_services.src.models.texts.languages import Language
from rooms_shared_services.src.storage.dynamodb import DynamodbStorageClient
from rooms_shared_services.src.translation.client import TranslationClient
from rooms_shared_services.src.translation.providers.aws import AWSTranslationProvider


class ProductAttributeClient(object):
    def __init__(
        self,
        storage_client: DynamodbStorageClient,
        name_language_indexname: str,
    ) -> None:
        """Set Attributes.

        Args:
            storage_client (DynamodbStorageClient): _description_
            name_language_indexname (str): _description_
        """
        self.storage_client = storage_client
        self.name_language_indexname = name_language_indexname

    def retrieve_attribute(self, attr_id: UUID, language: Language):
        key = {"attr_id": attr_id, "attr_language": language}
        attr_data = self.storage_client.retrieve(key=key)
        return ProductAttribute.validate_dynamodb_item(attr_data)

    def save_attribute(self, product_attribute: ProductAttribute) -> dict:
        table_item = product_attribute.dynamodb_dump(exclude_unset=False)
        print("table_item: {}".format(table_item))
        return self.storage_client.create(table_item=table_item)

    def create_attribute(self, attr_name: str, attr_language: Language, attr_terms: list[str]) -> dict:
        product_attribute = ProductAttribute(attr_name=attr_name, attr_language=attr_language, attr_terms=attr_terms)
        return self.save_attribute(product_attribute=product_attribute)

    def create_all(self, attr_dict: dict):
        for attr_name in attr_dict:
            attr_params = {"attr_name": attr_name, "attr_language": Language.en, "attr_terms": attr_dict[attr_name]}
            self.create_attribute(**attr_params)

    def iterate_page_attributes(self, page: list[dict]):
        for prod_item in page:
            prod_item["attr_language"] = getattr(Language, prod_item["attr_language"]).value
            yield ProductAttribute.validate_dynamodb_item(prod_item)

    def iterate_attributes(self, language: Language = Language.en):
        filter_by = {"attr_language": language.name}
        for page in self.storage_client.get_by_pages(filter_by=filter_by):
            yield from self.iterate_page_attributes(page)

    def create_translated_attribute(self, product_attribute: ProductAttribute, translation: dict, language: Language):
        model_params = {
            "id": product_attribute.id,
            "attr_name": translation["attr_name"],
            "attr_terms": translation["attr_terms"],
            "attr_language": language.value,
        }
        translated_attribute = ProductAttribute.model_validate(model_params)
        self.save_attribute(product_attribute=translated_attribute)

    def translate_all(self, language: Language):
        for product_attribute in self.iterate_attributes():
            translation = self.translate_attribute(product_attribute=product_attribute, language=language)
            self.create_translated_attribute(
                product_attribute=product_attribute,
                translation=translation,
                language=language,
            )

    def translate_name(
        self,
        product_attribute: ProductAttribute,
        language: Language,
        translations: dict,
        translation_client: TranslationClient,
    ):
        translations["attr_name"] = getattr(
            translation_client.bulk_translate(
                source_language=Language.en,
                target_languages=[language],
                txt=product_attribute.attr_name,
            ),
            language.name,
        )

    def translate_terms(
        self,
        product_attribute: ProductAttribute,
        language: Language,
        translations: dict,
        translation_client: TranslationClient,
    ):
        translated_terms = []
        for term in product_attribute.attr_terms:
            translated_terms.append(
                getattr(
                    translation_client.bulk_translate(
                        source_language=Language.en,
                        target_languages=[language],
                        txt=term,
                    ),
                    language.name,
                ),
            )
        translations["attr_terms"] = translated_terms

    def translate_attribute(self, product_attribute: ProductAttribute, language: Language):
        translations = {}
        provider = AWSTranslationProvider()
        translation_client = TranslationClient(provider=provider)
        self.translate_name(
            product_attribute=product_attribute,
            language=language,
            translations=translations,
            translation_client=translation_client,
        )
        self.translate_terms(
            product_attribute=product_attribute,
            language=language,
            translations=translations,
            translation_client=translation_client,
        )
        return translations



from datetime import datetime
from typing import Any
from uuid import UUID

from pydantic import HttpUrl, field_validator

from rooms_shared_services.src.models.relations import CurrencyCode
from rooms_shared_services.src.models.texts.translations import TextTranslations
from rooms_shared_services.src.storage.models import UNSET, BaseDynamodbModel


class ProductDescription(BaseDynamodbModel):
    language_code: str
    full: str | None = None
    short: str | None = None
    translation_provider: str | None = None
    approved: bool | None = None


class ProductBrand(BaseDynamodbModel):
    id: UUID | None = None
    name: str | None = None
    website: HttpUrl | None = None
    catalog_pdf: HttpUrl | None = None
    description: str | None = None
    logo: HttpUrl | None = None


class ProductCollection(BaseDynamodbModel):
    id: UUID | None = None
    name: str | None = None
    catalog_pdf: HttpUrl | None = None
    description: str | None = None
    logo: HttpUrl | None = None


class ColorAttributes(BaseDynamodbModel):
    main_color: str | None = None
    main_color_image: HttpUrl | None = None
    color: str | None = None
    color_image: HttpUrl | None = None
    upholstery_color: str | None = None
    upholstery_color_image: HttpUrl | None = None
    front_color: str | None = None
    front_color_image: HttpUrl | None = None
    cabinet_color: str | None = None
    cabinet_color_image: HttpUrl | None = None


class ProductAttributes(BaseDynamodbModel):
    furniture_types: list[str]
    country_of_origin: list[str]
    materials: list[str]
    delivery_term: list[str]


class ImageSet(BaseDynamodbModel):
    small: HttpUrl | None = None
    medium: HttpUrl | None = None
    large: HttpUrl | None = None


class PackagePack(BaseDynamodbModel):
    ean: str | None = None
    weight: float | None = None
    length: float | None = None
    packNum: int | None = None  # noqa: N815
    height: float | None = None
    width: float | None = None


class StoreVariantRelatedValues(BaseDynamodbModel):
    default: str | int | datetime


class GeoRelatedValues(BaseDynamodbModel):
    default: StoreVariantRelatedValues
    israel: StoreVariantRelatedValues | None = None
    uk: StoreVariantRelatedValues | None = None


class LanguageRelatedValues(BaseDynamodbModel):
    default: GeoRelatedValues
    hebrew: GeoRelatedValues | None = None
    english: GeoRelatedValues | None = None
    russian: GeoRelatedValues | None = None


class RelatedValues(BaseDynamodbModel):
    wc: LanguageRelatedValues


class PriceValue(BaseDynamodbModel):
    currency: CurrencyCode | UNSET = "UNSET"
    value: float | UNSET = "UNSET"  # noqa: WPS110

    @property
    def valid(self):
        return not (self.currency == "UNSET" or self.value == "UNSET")


class PriceValueCollection(BaseDynamodbModel):
    wc_israel_general_ils: PriceValue | None | UNSET = "UNSET"


class ProductSourceItem(BaseDynamodbModel):
    id: UUID | UNSET = "UNSET"
    product_group: str | UNSET = "UNSET"
    category: str | None | UNSET = "UNSET"
    room_category: str | None | UNSET = "UNSET"
    original_ident: str | None | UNSET = "UNSET"
    ident_code: str | None | UNSET = "UNSET"
    name: str | None | UNSET = "UNSET"
    slug: str | None | UNSET = "UNSET"
    created_at: datetime | None | UNSET = "UNSET"
    modified_at: datetime | None | UNSET = "UNSET"
    original_description_full: str | None | UNSET = "UNSET"
    original_description_short: str | None | UNSET = "UNSET"
    original_description_language_code: str | None | UNSET = "UNSET"
    origin_country: str | None | UNSET = "UNSET"
    pickup_location: str | None | UNSET = "UNSET"
    pickup_address: str | None | UNSET = "UNSET"
    shipment_term_code: str | None | UNSET = "UNSET"
    catalog_pdf: HttpUrl | None | UNSET = "UNSET"
    pickup_downtime: int | None | UNSET = "UNSET"
    brand_catalog_pages: list[int] | None | UNSET = "UNSET"
    collection_catalog_pages: list[int] | None | UNSET = "UNSET"
    brand_website_links: list[HttpUrl] | None | UNSET = "UNSET"
    sku: str | None | UNSET = "UNSET"
    wc_sku: str | None | UNSET = "UNSET"
    gross_weight: float | None | UNSET = "UNSET"
    net_weight: float | None | UNSET = "UNSET"
    height: float | None | UNSET = "UNSET"
    width: float | None | UNSET = "UNSET"
    depth: float | None | UNSET = "UNSET"
    related_ids: list[UUID] | None | UNSET = "UNSET"
    upsell_ids: list[UUID] | None | UNSET = "UNSET"
    cross_sell_ids: list[UUID] | None | UNSET = "UNSET"
    tags: list[str] | None | UNSET = "UNSET"
    image_sets: list[ImageSet] | None | UNSET = "UNSET"
    features: dict | None | UNSET = "UNSET"

    product_attributes: ProductAttributes | None | UNSET = "UNSET"
    color_attributes: ColorAttributes | None | UNSET = "UNSET"
    brand: ProductBrand | None | UNSET = "UNSET"
    collection: ProductCollection | None | UNSET = "UNSET"
    ean_GTIN: str | None | UNSET = "UNSET"  # noqa: N815
    qty_of_boxes: int | None | UNSET = "UNSET"
    brand_code: str | None | UNSET = "UNSET"
    volume_m3: float | None | UNSET = "UNSET"
    package_packs: list[PackagePack] | None | UNSET = "UNSET"
    qty_per_box: int | None | UNSET = "UNSET"
    ean: str | None | UNSET = "UNSET"
    withdrawn: bool | None | UNSET = "UNSET"
    pcn: str | None | UNSET = "UNSET"
    categories_raw: list[str] | None | UNSET = "UNSET"
    wc_categories: list[str] | None | UNSET = "UNSET"
    published: RelatedValues | None | UNSET = "UNSET"
    name_translations: TextTranslations | None | UNSET = "UNSET"
    full_description_translations: TextTranslations | None | UNSET = "UNSET"
    short_description_translations: TextTranslations | None | UNSET = "UNSET"
    price_value_collection: PriceValueCollection | None | UNSET = "UNSET"
    price_base: PriceValue | None | UNSET = "UNSET"

    furniture_types: list[str] | None | UNSET = "UNSET"
    country_of_origin: list[str] | None | UNSET = "UNSET"
    materials: list[str] | None | UNSET = "UNSET"
    delivery_term: list[str] | None | UNSET = "UNSET"

    @field_validator(
        "brand_catalog_pages",
        "collection_catalog_pages",
        "brand_website_links",
        "related_ids",
        "upsell_ids",
        "cross_sell_ids",
        "tags",
        "image_sets",
        "package_packs",
        "categories_raw",
        mode="before",
    )
    @classmethod
    def skip_invalid_lists(cls, item_value: Any) -> str:
        if not isinstance(item_value, list) and item_value is not None:
            return "UNSET"
        return item_value



from decimal import Decimal
from enum import Enum
from typing import Any, Literal
from uuid import uuid4

from pydantic import BaseModel

UNSET = Literal["UNSET"]


class BaseDynamodbModel(BaseModel):
    @classmethod
    def create_with_id(cls, **attributes):
        model_id = uuid4()
        value_dict = {"id": model_id, **attributes}
        return cls.model_validate(value_dict)

    def convert_value(self, item_value, exclude_unset: bool):
        match item_value:
            case BaseDynamodbModel():
                converted_item_value = item_value.dynamodb_dump(exclude_unset=exclude_unset)
            case int():
                converted_item_value = Decimal(item_value)
            case float():
                converted_item_value = Decimal(str(item_value))
            case dict():
                converted_item_value = self.convert_dict(item_value, exclude_unset=exclude_unset)
            case list():
                converted_item_value = [
                    self.convert_value(item_value_item, exclude_unset=exclude_unset) for item_value_item in item_value
                ]
            case Enum():
                converted_item_value = item_value.name
            case _:
                converted_item_value = str(item_value)
        return converted_item_value

    def convert_dict(self, item_dict: dict, exclude_unset: bool):
        return {
            item_key: self.convert_value(item_value, exclude_unset=exclude_unset)
            for (item_key, item_value) in item_dict.items()
        }

    def dynamodb_dump(self, exclude_unset: bool = True, include: set[str] | None = None):
        item_dict = self.model_dump(exclude_unset=exclude_unset, include=include)
        return self.convert_dict(item_dict=item_dict, exclude_unset=exclude_unset)

    @classmethod
    def validate_value(cls, item_value: Any) -> Any:
        match item_value:
            case list():
                validated_value = [cls.validate_value(item_value=product_elem) for product_elem in item_value]
            case "None":
                validated_value = None
            case dict():
                return {
                    elem_key: cls.validate_value(item_value=elem_value) for elem_key, elem_value in item_value.items()
                }
            case _:
                validated_value = item_value
        return validated_value

    @classmethod
    def validate_dynamodb_item(cls, data_dict: dict):
        key_value_dict = {}
        for product_key, product_value in data_dict.items():
            validated_key = product_key
            match product_key:
                case _:
                    pass
            validated_value = cls.validate_value(item_value=product_value)
            key_value_dict.update({validated_key: validated_value})

        return cls.model_validate(key_value_dict)



from datetime import datetime, timezone
from decimal import Decimal

import boto3
from boto3.dynamodb.conditions import Key

from rooms_shared_services.src.delayed_tasks.abstract import (
    AbstractDelayedTaskBulkStorageClient,
    AbstractDelayedTaskSingleStorageClient,
)
from rooms_shared_services.src.delayed_tasks.models import DelayedTask, TaskStatus
from rooms_shared_services.src.settings.delayed_tasks import Settings as DelayedTasksSettings
from rooms_shared_services.src.storage.dynamodb import DynamodbStorageClient

delayed_task_settings = DelayedTasksSettings()
dynamodb_resource = boto3.resource("dynamodb", region_name=delayed_task_settings.region_name)


class DynamodbDelayedTaskSingleClient(AbstractDelayedTaskSingleStorageClient):
    def __init__(self, db_client: DynamodbStorageClient, task_id: str, attempt_number: int | None = None):
        """Assign attribues.

        Args:
            db_client (DynamodbStorageClient): _description_
            task_id (str): _description_
            attempt_number (int | None, optional): _description_. Defaults to None.
        """
        self.db_client = db_client
        match attempt_number:
            case None:
                query_resp = self.db_client.table.query(KeyConditionExpression=Key("task_id").eq(task_id))
                task_items = query_resp["Items"]
                task_items.sort(key=self.sort_by_attempt_number)
                task_item = task_items[-1]
            case _:
                get_resp = self.db_client.table.get_item(
                    Key={"task_id": task_id, "attempt_number": Decimal(attempt_number)},
                )
                task_item = get_resp["Item"]
        self.task_item_key = {"task_id": task_item["task_id"], "attempt_number": task_item["attempt_number"]}

    @staticmethod
    def sort_by_attempt_number(attempt):
        return attempt["attempt_number"]

    def retrieve(self):
        return self.db_client.retrieve(key=self.task_item_key)

    def update_status(self, status: TaskStatus):
        attribute_updates = {"task_status": status}
        return self.db_client.update(key=self.task_item_key, attribute_updates=attribute_updates)


class DynamodbDelayedTaskBulkStorageClient(AbstractDelayedTaskBulkStorageClient):
    def __init__(self, db_client: DynamodbStorageClient, task_variant: str) -> None:
        """Assign db_client.

        Args:
            db_client (DynamodbStorageClient): _description_
            task_variant (str): _description_
        """
        self.db_client = db_client
        super().__init__(task_variant=task_variant)

    def get_task_items(self, status: TaskStatus) -> list[DelayedTask]:
        filter_params = {"task_status": status, "task_variant": self.task_variant}
        task_items = self.db_client.bulk_get(filter_params=filter_params)
        return [DelayedTask.model_validate(task_item) for task_item in task_items]

    def bulk_update_status(self, key_list: list[dict], status: TaskStatus) -> list[dict]:
        match status:
            case "scheduled":
                timestamp_attr = "scheduled_at"
            case "pending":
                timestamp_attr = "pending_at"
            case "processing":
                timestamp_attr = "processed_at"
            case "succeded":
                timestamp_attr = "succeded_at"
            case "failed":
                timestamp_attr = "failed_at"
            case _:
                raise ValueError("Invalid task status")
        timestamp = datetime.now(tz=timezone.utc).isoformat()
        attribute_update_list = [{"task_status": status, timestamp_attr: timestamp}] * len(key_list)  # noqa: WPS435
        return self.db_client.bulk_update(keys=key_list, attribute_updates_list=attribute_update_list)



from random import choice
from typing import Any, Literal, TypeVar, get_args
from uuid import uuid4

from faker import Faker
from pydantic import BaseModel
from pydantic.fields import FieldInfo

fake = Faker()


AnnotationName = Literal["str", "int", "float"]


def get_fake_value(field_info: FieldInfo) -> Any:  # noqa: WPS212
    """Get fake value for a field info based on annotation.

    Args:
        field_info (FieldInfo): pydantic FieldInfo instance

    Raises:
        ValueError: When Invalid FieldInfo provided

    Returns:
        Any: any fake value
    """
    if not field_info.annotation:
        return None
    name = field_info.annotation.__name__
    match name:
        case "str":
            return fake.pystr()
        case "int":
            return fake.pyint()
        case "UUID":
            return uuid4()
        case "Literal":
            return choice(get_args(field_info.annotation))
        case "bool":
            return choice((True, False))
        case _:
            raise ValueError("Invalid field annotation for generating fake value: {}".format(name))


T = TypeVar("T", bound="BaseModel")


def make_fake(model_type: type[T]) -> T:
    """Make fake model instance with arbitrary values.

    Args:
        model_type (type[T]): Any BaseModel subclass instance

    Returns:
        T: created fake model instance
    """
    fields = model_type.model_fields
    value_dict = {}
    for key in fields:
        field_info = fields[key]
        fake_value = get_fake_value(field_info=field_info)
        value_dict.update({key: fake_value})
    return model_type.model_validate(value_dict)
